@@ -2,9 +2,27 @@ import pytest
 
 
 @pytest.fixture
-def project(project):
-    project.write_file("requirements/setup.txt", contents="setup-requires-pkg")
-    project.write_file("requirements/base.txt", contents="install-requires-pkg")
+def project(project, pypi_server_port):
+    project.write_file(
+        "requirements/setup.txt",
+        contents="""
+        -i http://localhost:{}
+
+        setup-requires-pkg
+        """.format(
+            pypi_server_port
+        ),
+    )
+    project.write_file(
+        "requirements/base.txt",
+        contents="""
+        --index-url=http://localhost:{}
+
+        install-requires-pkg
+        """.format(
+            pypi_server_port
+        ),
+    )
     project.write_file("requirements/docs.txt", contents="docs-extras-require-pkg")
     project.write_file("requirements/cli.txt", contents="cli-extras-require-pkg")
     project.write_file("requirements/tests.txt", contents="tests-require-pkg")
@@ -23,13 +41,13 @@ def project(project):
 @pytest.mark.parametrize("build", ["sdist", "bdist_wheel"])
 def test_build_package(project, build):
     ret = project.sys_exec_run("setup.py", build)
-    assert ret.exitcode == 0
+    assert ret.returncode == 0
     package = project.get_generated_dist()
     with project.virtualenv() as venv:
         installed_packages = venv.get_installed_packages()
         assert "install-requires-pkg" not in installed_packages
         ret = venv.install(str(package))
-        assert ret.exitcode == 0, ret
+        assert ret.returncode == 0, ret
         installed_packages = venv.get_installed_packages()
         assert "install-requires-pkg" in installed_packages
 
@@ -37,7 +55,7 @@ def test_build_package(project, build):
         installed_packages = venv.get_installed_packages()
         assert "docs-extras-require-pkg" not in installed_packages
         ret = venv.install("{}[docs]".format(package))
-        assert ret.exitcode == 0, ret
+        assert ret.returncode == 0, ret
         installed_packages = venv.get_installed_packages()
         assert "docs-extras-require-pkg" in installed_packages
 
@@ -46,5 +64,5 @@ def test_build_package(project, build):
         assert "cli-extras-require-pkg" not in installed_packages
         ret = venv.install("{}[cli]".format(package))
         installed_packages = venv.get_installed_packages()
-        assert ret.exitcode == 0, ret
+        assert ret.returncode == 0, ret
         assert "cli-extras-require-pkg" in installed_packages
